@@ -68,6 +68,17 @@ class TimeFrames:
     def all(self) -> List[TimeFrame]:
         return list(self.__frames)
 
+    def __str__(self):
+        txt = "start .......... end ................ pv power total ........... power per hour\n"
+        for frame in sorted(self.__frames, key=lambda frame: frame.start_time, reverse=False):
+            power = str(round(frame.power_total))
+            power_per_hour = ", ".join([str(round(power)) for power in frame.hourly_power])
+            txt += frame.start_time.strftime("%d %b, %H:%M") + " .. " + \
+                   frame.end_time.strftime("%d %b, %H:%M") + " " + \
+                   "".join(["."] * (15 - len(power))) + " " + power + " watt" + " " + \
+                   "".join(["."] * (25 - len(power_per_hour))) + " " + power_per_hour + "\n"
+        return txt
+
 
 class Next24hours:
 
@@ -115,13 +126,21 @@ class Next24hours:
         return TimeFrames(frames)
 
     def __str__(self):
-        txt = ""
+        txt = "local time .......... pv power ..... irradiance ..... visibility .... fog probab. .... cloud cover\n"
         for time in list(self.predictions.keys())[:24]:
-            txt += time.strftime("%d.%m %H:%M") + ": " + str(self.predictions[time].power_watt) +\
-                   "  (irradiance=" + str(self.predictions[time].irradiance) + \
-                   ", visibility=" + str(self.predictions[time].visibility) + \
-                   ", probability_for_fog=" + str(self.predictions[time].probability_for_fog) + \
-                   ", cloud_cover=" + str(self.predictions[time].cloud_cover) + ")\n"
+            power = str(round(self.predictions[time].power_watt))
+            irradiance = str(round(self.predictions[time].irradiance))
+            visibility = str(round(self.predictions[time].visibility))
+            probability_for_fog = str(round(self.predictions[time].probability_for_fog))
+            cloud_cover = str(round(self.predictions[time].cloud_cover))
+            txt += time.strftime("%d %b, %H:%M") + " " + \
+                   "".join(["."] * (10 - len(power))) + " " + power + " watt " + \
+                   "".join(["."] * (15 - len(irradiance))) + " " + irradiance + " " + \
+                   "".join(["."] * (15 - len(visibility))) + " " + visibility + " " + \
+                   "".join(["."] * (15 - len(probability_for_fog))) + " " + probability_for_fog +  " " + \
+                   "".join(["."] * (15 - len(cloud_cover))) + " " + cloud_cover + "\n"
         return txt
 
 
+
+
